# coding: utf-8

"""
    fine-tuning-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import re  # noqa: F401
import io
import warnings

from pydantic.v1 import validate_arguments, ValidationError

from typing_extensions import Annotated
from pydantic.v1 import Field, StrictInt, StrictStr, conlist

from typing import Any, Dict, Optional

from clients.fine_tuning.models.create_tune_request import CreateTuneRequest
from clients.fine_tuning.models.list_tunes_response import ListTunesResponse
from clients.fine_tuning.models.tune import Tune

from clients.fine_tuning.api_client import ApiClient
from clients.fine_tuning.api_response import ApiResponse
from clients.fine_tuning.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class TuneApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def cancel_tune_v1_tune_tune_id_cancel_post(self, tune_id : Annotated[StrictStr, Field(..., description="The ID of the tune to cancel.")], **kwargs) -> Tune:  # noqa: E501
        """Cancel Tune  # noqa: E501

        Cancel the specified tune.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.cancel_tune_v1_tune_tune_id_cancel_post(tune_id, async_req=True)
        >>> result = thread.get()

        :param tune_id: The ID of the tune to cancel. (required)
        :type tune_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Tune
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the cancel_tune_v1_tune_tune_id_cancel_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.cancel_tune_v1_tune_tune_id_cancel_post_with_http_info(tune_id, **kwargs)  # noqa: E501

    @validate_arguments
    def cancel_tune_v1_tune_tune_id_cancel_post_with_http_info(self, tune_id : Annotated[StrictStr, Field(..., description="The ID of the tune to cancel.")], **kwargs) -> ApiResponse:  # noqa: E501
        """Cancel Tune  # noqa: E501

        Cancel the specified tune.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.cancel_tune_v1_tune_tune_id_cancel_post_with_http_info(tune_id, async_req=True)
        >>> result = thread.get()

        :param tune_id: The ID of the tune to cancel. (required)
        :type tune_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Tune, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'tune_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cancel_tune_v1_tune_tune_id_cancel_post" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['tune_id']:
            _path_params['tune_id'] = _params['tune_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "Tune",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/v1/tune/{tune_id}/cancel', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def create_tune_v1_tune_post(self, create_tune_request : CreateTuneRequest, **kwargs) -> Tune:  # noqa: E501
        """Create Tune  # noqa: E501

        Spawn a tune.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_tune_v1_tune_post(create_tune_request, async_req=True)
        >>> result = thread.get()

        :param create_tune_request: (required)
        :type create_tune_request: CreateTuneRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Tune
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the create_tune_v1_tune_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.create_tune_v1_tune_post_with_http_info(create_tune_request, **kwargs)  # noqa: E501

    @validate_arguments
    def create_tune_v1_tune_post_with_http_info(self, create_tune_request : CreateTuneRequest, **kwargs) -> ApiResponse:  # noqa: E501
        """Create Tune  # noqa: E501

        Spawn a tune.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_tune_v1_tune_post_with_http_info(create_tune_request, async_req=True)
        >>> result = thread.get()

        :param create_tune_request: (required)
        :type create_tune_request: CreateTuneRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Tune, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'create_tune_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_tune_v1_tune_post" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['create_tune_request'] is not None:
            _body_params = _params['create_tune_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '201': "Tune",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/v1/tune', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def delete_tune_v1_tune_tune_id_delete(self, tune_id : Annotated[StrictStr, Field(..., description="The ID of the tune.")], **kwargs) -> object:  # noqa: E501
        """Delete Tune  # noqa: E501

        Delete the specified tune.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_tune_v1_tune_tune_id_delete(tune_id, async_req=True)
        >>> result = thread.get()

        :param tune_id: The ID of the tune. (required)
        :type tune_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: object
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the delete_tune_v1_tune_tune_id_delete_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.delete_tune_v1_tune_tune_id_delete_with_http_info(tune_id, **kwargs)  # noqa: E501

    @validate_arguments
    def delete_tune_v1_tune_tune_id_delete_with_http_info(self, tune_id : Annotated[StrictStr, Field(..., description="The ID of the tune.")], **kwargs) -> ApiResponse:  # noqa: E501
        """Delete Tune  # noqa: E501

        Delete the specified tune.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_tune_v1_tune_tune_id_delete_with_http_info(tune_id, async_req=True)
        >>> result = thread.get()

        :param tune_id: The ID of the tune. (required)
        :type tune_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(object, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'tune_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_tune_v1_tune_tune_id_delete" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['tune_id']:
            _path_params['tune_id'] = _params['tune_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "object",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/v1/tune/{tune_id}', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def get_tune_v1_tune_tune_id_get(self, tune_id : Annotated[StrictStr, Field(..., description="The ID of the tune.")], **kwargs) -> Tune:  # noqa: E501
        """Get Tune  # noqa: E501

        Get the specific tune.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_tune_v1_tune_tune_id_get(tune_id, async_req=True)
        >>> result = thread.get()

        :param tune_id: The ID of the tune. (required)
        :type tune_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Tune
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the get_tune_v1_tune_tune_id_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.get_tune_v1_tune_tune_id_get_with_http_info(tune_id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_tune_v1_tune_tune_id_get_with_http_info(self, tune_id : Annotated[StrictStr, Field(..., description="The ID of the tune.")], **kwargs) -> ApiResponse:  # noqa: E501
        """Get Tune  # noqa: E501

        Get the specific tune.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_tune_v1_tune_tune_id_get_with_http_info(tune_id, async_req=True)
        >>> result = thread.get()

        :param tune_id: The ID of the tune. (required)
        :type tune_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Tune, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'tune_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_tune_v1_tune_tune_id_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['tune_id']:
            _path_params['tune_id'] = _params['tune_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "Tune",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/v1/tune/{tune_id}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def health_check_health_get(self, **kwargs) -> object:  # noqa: E501
        """Health Check  # noqa: E501

        Healthcheck server endpoint.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.health_check_health_get(async_req=True)
        >>> result = thread.get()

        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: object
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the health_check_health_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.health_check_health_get_with_http_info(**kwargs)  # noqa: E501

    @validate_arguments
    def health_check_health_get_with_http_info(self, **kwargs) -> ApiResponse:  # noqa: E501
        """Health Check  # noqa: E501

        Healthcheck server endpoint.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.health_check_health_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(object, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method health_check_health_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "object",
        }

        return self.api_client.call_api(
            '/health', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def list_tunes_v1_tunes_get(self, offset : Annotated[Optional[StrictInt], Field(description="Offset into the results.")] = None, limit : Annotated[Optional[StrictInt], Field(description="The max number of results to be shown (limit 100).")] = None, name : Annotated[Optional[StrictStr], Field(description="The name of the tune to filter on.")] = None, tune_type : Annotated[Optional[StrictStr], Field(description="The type of the tune to filter on.")] = None, base_checkpoint_id : Annotated[Optional[StrictStr], Field(description="The base checkpoint ID for the LoRA tune.")] = None, trigger_words : Annotated[Optional[conlist(StrictStr)], Field(description="The trigger words to filter on.")] = None, engine : Annotated[Optional[StrictStr], Field(description="The engine type.")] = None, **kwargs) -> ListTunesResponse:  # noqa: E501
        """List Tunes  # noqa: E501

        List all tunes owned by the current user.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_tunes_v1_tunes_get(offset, limit, name, tune_type, base_checkpoint_id, trigger_words, engine, async_req=True)
        >>> result = thread.get()

        :param offset: Offset into the results.
        :type offset: int
        :param limit: The max number of results to be shown (limit 100).
        :type limit: int
        :param name: The name of the tune to filter on.
        :type name: str
        :param tune_type: The type of the tune to filter on.
        :type tune_type: str
        :param base_checkpoint_id: The base checkpoint ID for the LoRA tune.
        :type base_checkpoint_id: str
        :param trigger_words: The trigger words to filter on.
        :type trigger_words: List[str]
        :param engine: The engine type.
        :type engine: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ListTunesResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the list_tunes_v1_tunes_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.list_tunes_v1_tunes_get_with_http_info(offset, limit, name, tune_type, base_checkpoint_id, trigger_words, engine, **kwargs)  # noqa: E501

    @validate_arguments
    def list_tunes_v1_tunes_get_with_http_info(self, offset : Annotated[Optional[StrictInt], Field(description="Offset into the results.")] = None, limit : Annotated[Optional[StrictInt], Field(description="The max number of results to be shown (limit 100).")] = None, name : Annotated[Optional[StrictStr], Field(description="The name of the tune to filter on.")] = None, tune_type : Annotated[Optional[StrictStr], Field(description="The type of the tune to filter on.")] = None, base_checkpoint_id : Annotated[Optional[StrictStr], Field(description="The base checkpoint ID for the LoRA tune.")] = None, trigger_words : Annotated[Optional[conlist(StrictStr)], Field(description="The trigger words to filter on.")] = None, engine : Annotated[Optional[StrictStr], Field(description="The engine type.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """List Tunes  # noqa: E501

        List all tunes owned by the current user.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_tunes_v1_tunes_get_with_http_info(offset, limit, name, tune_type, base_checkpoint_id, trigger_words, engine, async_req=True)
        >>> result = thread.get()

        :param offset: Offset into the results.
        :type offset: int
        :param limit: The max number of results to be shown (limit 100).
        :type limit: int
        :param name: The name of the tune to filter on.
        :type name: str
        :param tune_type: The type of the tune to filter on.
        :type tune_type: str
        :param base_checkpoint_id: The base checkpoint ID for the LoRA tune.
        :type base_checkpoint_id: str
        :param trigger_words: The trigger words to filter on.
        :type trigger_words: List[str]
        :param engine: The engine type.
        :type engine: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ListTunesResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'offset',
            'limit',
            'name',
            'tune_type',
            'base_checkpoint_id',
            'trigger_words',
            'engine'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_tunes_v1_tunes_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('offset') is not None:  # noqa: E501
            _query_params.append(('offset', _params['offset']))

        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        if _params.get('name') is not None:  # noqa: E501
            _query_params.append(('name', _params['name']))

        if _params.get('tune_type') is not None:  # noqa: E501
            _query_params.append(('tune_type', _params['tune_type']))

        if _params.get('base_checkpoint_id') is not None:  # noqa: E501
            _query_params.append(('base_checkpoint_id', _params['base_checkpoint_id']))

        if _params.get('trigger_words') is not None:  # noqa: E501
            _query_params.append(('trigger_words', _params['trigger_words']))
            _collection_formats['trigger_words'] = 'multi'

        if _params.get('engine') is not None:  # noqa: E501
            _query_params.append(('engine', _params['engine']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "ListTunesResponse",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/v1/tunes', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import io
import json
import logging
import re
import ssl

from urllib.parse import urlencode, quote_plus
import urllib3

from clients.image_gen.exceptions import ApiException, UnauthorizedException, ForbiddenException, NotFoundException, ServiceException, ApiValueError, BadRequestException


logger = logging.getLogger(__name__)


class RESTResponse(io.IOBase):

    def __init__(self, resp) -> None:
        self.urllib3_response = resp
        self.status = resp.status
        self.reason = resp.reason
        self.data = resp.data

    def getheaders(self):
        """Returns a dictionary of the response headers."""
        return self.urllib3_response.headers

    def getheader(self, name, default=None):
        """Returns a given response header."""
        return self.urllib3_response.headers.get(name, default)


class RESTClientObject:

    def __init__(self, configuration, pools_size=4, maxsize=None) -> None:
        # urllib3.PoolManager will pass all kw parameters to connectionpool
        # https://github.com/shazow/urllib3/blob/f9409436f83aeb79fbaf090181cd81b784f1b8ce/urllib3/poolmanager.py#L75  # noqa: E501
        # https://github.com/shazow/urllib3/blob/f9409436f83aeb79fbaf090181cd81b784f1b8ce/urllib3/connectionpool.py#L680  # noqa: E501
        # maxsize is the number of requests to host that are allowed in parallel  # noqa: E501
        # Custom SSL certificates and client certificates: http://urllib3.readthedocs.io/en/latest/advanced-usage.html  # noqa: E501

        # cert_reqs
        if configuration.verify_ssl:
            cert_reqs = ssl.CERT_REQUIRED
        else:
            cert_reqs = ssl.CERT_NONE

        addition_pool_args = {}
        if configuration.assert_hostname is not None:
            addition_pool_args['assert_hostname'] = configuration.assert_hostname  # noqa: E501

        if configuration.retries is not None:
            addition_pool_args['retries'] = configuration.retries

        if configuration.tls_server_name:
            addition_pool_args['server_hostname'] = configuration.tls_server_name


        if configuration.socket_options is not None:
            addition_pool_args['socket_options'] = configuration.socket_options

        if maxsize is None:
            if configuration.connection_pool_maxsize is not None:
                maxsize = configuration.connection_pool_maxsize
            else:
                maxsize = 4

        # https pool manager
        if configuration.proxy:
            self.pool_manager = urllib3.ProxyManager(
                num_pools=pools_size,
                maxsize=maxsize,
                cert_reqs=cert_reqs,
                ca_certs=configuration.ssl_ca_cert,
                cert_file=configuration.cert_file,
                key_file=configuration.key_file,
                proxy_url=configuration.proxy,
                proxy_headers=configuration.proxy_headers,
                **addition_pool_args
            )
        else:
            self.pool_manager = urllib3.PoolManager(
                num_pools=pools_size,
                maxsize=maxsize,
                cert_reqs=cert_reqs,
                ca_certs=configuration.ssl_ca_cert,
                cert_file=configuration.cert_file,
                key_file=configuration.key_file,
                **addition_pool_args
            )

    def request(self, method, url, query_params=None, headers=None,
                body=None, post_params=None, _preload_content=True,
                _request_timeout=None):
        """Perform requests.

        :param method: http request method
        :param url: http request url
        :param query_params: query parameters in the url
        :param headers: http request headers
        :param body: request json body, for `application/json`
        :param post_params: request post parameters,
                            `application/x-www-form-urlencoded`
                            and `multipart/form-data`
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        """
        method = method.upper()
        assert method in ['GET', 'HEAD', 'DELETE', 'POST', 'PUT',
                          'PATCH', 'OPTIONS']

        if post_params and body:
            raise ApiValueError(
                "body parameter cannot be used with post_params parameter."
            )

        post_params = post_params or {}
        headers = headers or {}
        # url already contains the URL query string
        # so reset query_params to empty dict
        query_params = {}

        timeout = None
        if _request_timeout:
            if isinstance(_request_timeout, (int,float)):  # noqa: E501,F821
                timeout = urllib3.Timeout(total=_request_timeout)
            elif (isinstance(_request_timeout, tuple) and
                  len(_request_timeout) == 2):
                timeout = urllib3.Timeout(
                    connect=_request_timeout[0], read=_request_timeout[1])

        try:
            # For `POST`, `PUT`, `PATCH`, `OPTIONS`, `DELETE`
            if method in ['POST', 'PUT', 'PATCH', 'OPTIONS', 'DELETE']:

                # no content type provided or payload is json
                if not headers.get('Content-Type') or re.search('json', headers['Content-Type'], re.IGNORECASE):
                    request_body = None
                    if body is not None:
                        request_body = json.dumps(body)
                    r = self.pool_manager.request(
                        method, url,
                        body=request_body,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers)
                elif headers['Content-Type'] == 'application/x-www-form-urlencoded':  # noqa: E501
                    r = self.pool_manager.request(
                        method, url,
                        fields=post_params,
                        encode_multipart=False,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers)
                elif headers['Content-Type'] == 'multipart/form-data':
                    # must del headers['Content-Type'], or the correct
                    # Content-Type which generated by urllib3 will be
                    # overwritten.
                    del headers['Content-Type']
                    r = self.pool_manager.request(
                        method, url,
                        fields=post_params,
                        encode_multipart=True,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers)
                # Pass a `string` parameter directly in the body to support
                # other content types than Json when `body` argument is
                # provided in serialized form
                elif isinstance(body, str) or isinstance(body, bytes):
                    request_body = body
                    r = self.pool_manager.request(
                        method, url,
                        body=request_body,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers)
                else:
                    # Cannot generate the request from given parameters
                    msg = """Cannot prepare a request message for provided
                             arguments. Please check that your arguments match
                             declared content type."""
                    raise ApiException(status=0, reason=msg)
            # For `GET`, `HEAD`
            else:
                r = self.pool_manager.request(method, url,
                                              fields={},
                                              preload_content=_preload_content,
                                              timeout=timeout,
                                              headers=headers)
        except urllib3.exceptions.SSLError as e:
            msg = "{0}\n{1}".format(type(e).__name__, str(e))
            raise ApiException(status=0, reason=msg)

        if _preload_content:
            r = RESTResponse(r)

            # log response body
            logger.debug("response body: %s", r.data)

        if not 200 <= r.status <= 299:
            if r.status == 400:
                raise BadRequestException(http_resp=r)

            if r.status == 401:
                raise UnauthorizedException(http_resp=r)

            if r.status == 403:
                raise ForbiddenException(http_resp=r)

            if r.status == 404:
                raise NotFoundException(http_resp=r)

            if 500 <= r.status <= 599:
                raise ServiceException(http_resp=r)

            raise ApiException(http_resp=r)

        return r

    def get_request(self, url, headers=None, query_params=None, _preload_content=True,
            _request_timeout=None):
        return self.request("GET", url,
                            headers=headers,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            query_params=query_params)

    def head_request(self, url, headers=None, query_params=None, _preload_content=True,
             _request_timeout=None):
        return self.request("HEAD", url,
                            headers=headers,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            query_params=query_params)

    def options_request(self, url, headers=None, query_params=None, post_params=None,
                body=None, _preload_content=True, _request_timeout=None):
        return self.request("OPTIONS", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def delete_request(self, url, headers=None, query_params=None, body=None,
               _preload_content=True, _request_timeout=None):
        return self.request("DELETE", url,
                            headers=headers,
                            query_params=query_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def post_request(self, url, headers=None, query_params=None, post_params=None,
             body=None, _preload_content=True, _request_timeout=None):
        return self.request("POST", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def put_request(self, url, headers=None, query_params=None, post_params=None,
            body=None, _preload_content=True, _request_timeout=None):
        return self.request("PUT", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def patch_request(self, url, headers=None, query_params=None, post_params=None,
              body=None, _preload_content=True, _request_timeout=None):
        return self.request("PATCH", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)



"""Generate examples for the ``octoai.service.Service`` subclass."""

from enum import Enum

from ..service import Service
from ..spec import parse_service
from .common import DocumentationGenerator, MustacheGenerationResult
from .curl import CurlDocumentationGenerator
from .python import PythonExampleGenerator


class DocumentationFormat(Enum):
    """The format of the documentation generated."""

    CURL = 1
    PYTHON = 2


def generate_usage_examples(
    service: Service, format: DocumentationFormat
) -> MustacheGenerationResult:
    """Generate examples for the specified service."""
    generator: DocumentationGenerator
    if format == DocumentationFormat.CURL:
        generator = CurlDocumentationGenerator()
    elif format == DocumentationFormat.PYTHON:
        generator = PythonExampleGenerator()
    else:
        raise ValueError(f"Invalid documentation format f{format}")

    return generator.generate(parse_service(type(service)))


__all__ = ["generate_usage_examples", "DocumentationFormat", "MustacheGenerationResult"]



# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr
from clients.asset_lake.models.asset_type import AssetType
from clients.asset_lake.models.transfer_api_type import TransferApiType

class CreateAssetRequest(BaseModel):
    """
    CreateAssetRequest
    """
    asset_type: AssetType = Field(...)
    data: Dict[str, Any] = Field(..., description="Data specific to the asset type; use the class named ${asset_type}Data")
    description: Optional[StrictStr] = Field(...)
    is_public: Optional[StrictBool] = Field(False, description="True if asset is public.")
    name: StrictStr = Field(..., description="Asset name.")
    transfer_api_type: Optional[TransferApiType] = None
    __properties = ["asset_type", "data", "description", "is_public", "name", "transfer_api_type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateAssetRequest:
        """Create an instance of CreateAssetRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateAssetRequest:
        """Create an instance of CreateAssetRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateAssetRequest.parse_obj(obj)

        _obj = CreateAssetRequest.parse_obj({
            "asset_type": obj.get("asset_type"),
            "data": obj.get("data"),
            "description": obj.get("description"),
            "is_public": obj.get("is_public") if obj.get("is_public") is not None else False,
            "name": obj.get("name"),
            "transfer_api_type": obj.get("transfer_api_type")
        })
        return _obj





"""Generate Python examples."""

from dataclasses import dataclass
from typing import Optional, Set, cast

from octoai.documenter.common import (
    ASTVisitor,
    Buffer,
    DocumentationGenerator,
    MustacheGenerationResult,
    MustacheTemplateRegistry,
)
from octoai.documenter.json import (
    ConciseTypeFormatter,
    JSONGeneratingVisitor,
    JSONTypeFormatter,
)
from octoai.spec import SetType
from octoai.spec.ast import (
    BaseType,
    Field,
    FieldInfo,
    InferenceService,
    MediaKind,
    MediaType,
    OptionalType,
    PrimitiveType,
    PydanticHttpUrlType,
    SpecialType,
)


class PythonTypeFormatter(JSONTypeFormatter):
    """Provide a Python-like display of the types."""

    @classmethod
    def format_primitive(
        cls, primitive_type: PrimitiveType, field_info: FieldInfo
    ) -> str:
        """Show a Python-like format for the primitive type."""
        if primitive_type.type == bool:
            return "<True|False>"
        else:
            return super().format_primitive(primitive_type, field_info)

    @classmethod
    def format_set(cls, set_type: SetType, field_info: FieldInfo) -> str:
        """Show a Python-like format for the set type."""
        value = "{set of "
        value += ConciseTypeFormatter.format(set_type.member_type, field_info)
        value += "}"

        return value

    @classmethod
    def format_media(cls, media_type: MediaType, field_info: FieldInfo) -> str:
        """Show a Python-like format for the special media types."""
        if media_type.kind == MediaKind.IMAGE:
            class_name = "Image"
        elif media_type.kind == MediaKind.VIDEO:
            class_name = "Video"
        elif media_type.kind == MediaKind.AUDIO:
            class_name = "Audio"
        else:
            raise ValueError(f"unknown media type {media_type}")

        return f"{class_name}.from_url(<url string>).model_dump()"

    @classmethod
    def format_optional(cls, optional_type: OptionalType, field_info: FieldInfo) -> str:
        """Format the optional type."""
        example_value = cls._get_example_value(field_info)
        if example_value is not None:
            return example_value

        value = ConciseTypeFormatter.format(optional_type.some_type, field_info)
        value += "|None"

        return value


@dataclass
class PythonMustacheGenerationResult(MustacheGenerationResult):
    """Special generation result type to support imports."""

    discovered_special_types: Set[SpecialType]


class PythonInputGenerator(JSONGeneratingVisitor):
    """Generates a Mustache template that when rendered looks like a Python dict.

    This additionally stores all the special types encountered and return a different
    ``PythonMustacheGenerationResult``.
    """

    def __init__(
        self,
        base_buffer: Optional[Buffer] = None,
        template_registry: Optional[MustacheTemplateRegistry] = None,
    ):
        super().__init__(PythonTypeFormatter, base_buffer, template_registry)
        self._discovered_special_types: Set[SpecialType] = set()

    def generate(self, field: Field) -> MustacheGenerationResult:
        """Generate a Mustache template. This is the entry point.

        :param field: the field to generate a Mustache template for.
        :return a ``PythonMustacheGenerationResult`` containing the template, hash,
            and encountered types.
        """
        self.visit_base(field.type, field.info)
        return PythonMustacheGenerationResult(
            generated_template=self._buffer.string,
            template_hash=self._template_registry.hash,
            discovered_special_types=self._discovered_special_types,
        )

    def visit_pydantic_http_url(
        self,
        pydantic_http_url_type: PydanticHttpUrlType,
        field_info: FieldInfo,
    ):
        """Visit the Pydantic HttpUrl type."""
        self._discovered_special_types.add(pydantic_http_url_type)
        super().visit_pydantic_http_url(pydantic_http_url_type, field_info)

    def visit_media(
        self,
        media_type: MediaType,
        field_info: FieldInfo,
    ):
        """Visit the media type."""
        self._discovered_special_types.add(media_type)
        super().visit_media(media_type, field_info)


@dataclass
class PythonOutputGenerationResult:
    """Generation result type for the output portion in the Python usage example."""

    generated_string: str
    discovered_special_types: Set[SpecialType]


class PythonOutputGenerator(ASTVisitor):
    """Generates the output portion in the Python usage example."""

    def __init__(
        self,
        base_buffer: Optional[Buffer] = None,
    ):
        super().__init__()
        self._buffer = Buffer() if base_buffer is None else base_buffer
        self._discovered_special_types: Set[SpecialType] = set()

    def generate(self, field: Field) -> PythonOutputGenerationResult:
        """Generate the output usage example. This is the entry point.

        :param field: the field to generate a usage example for.
        """
        self.visit_base(field.type, field.info)
        return PythonOutputGenerationResult(
            generated_string=self._buffer.string,
            discovered_special_types=self._discovered_special_types,
        )

    def visit_base(
        self,
        base_type: BaseType,
        field_info: FieldInfo,
    ):
        """Visit the base type.

        :param base_type: the base type.
        :param field_info: information about the field.
        """
        if isinstance(base_type, MediaType) or isinstance(base_type, SpecialType):
            super().visit_base(base_type, field_info)
            return

        self._buffer.print_string("output")

    def visit_pydantic_http_url(
        self,
        pydantic_http_url_type: PydanticHttpUrlType,
        field_info: FieldInfo,
    ):
        """Visit the Pydantic HttpUrl type.

        :param pydantic_http_url_type: the Pydantic HttpUrl type.
        :param field_info: information about the field.
        """
        self._discovered_special_types.add(pydantic_http_url_type)
        super().visit_pydantic_http_url(pydantic_http_url_type, field_info)

    def _media_kind_to_class_name(self, media_type: MediaKind) -> str:
        if media_type == MediaKind.IMAGE:
            return "Image"
        elif media_type == MediaKind.VIDEO:
            return "Video"
        elif media_type == MediaKind.AUDIO:
            return "Audio"
        else:
            raise ValueError(f"Unknown media type {media_type}")

    def visit_media(
        self,
        media_type: MediaType,
        field_info: FieldInfo,
    ):
        """Visit the media type.

        :param media_type: the media type.
        :param field_info: information about the field.
        """
        self._discovered_special_types.add(media_type)
        super().visit_media(media_type, field_info)

        self._buffer.print_string(
            f"{self._media_kind_to_class_name(media_type.kind)}"
            f".from_endpoint_response(response)"
        )


class PythonExampleGenerator(DocumentationGenerator):
    """Generate Python SDK documentation."""

    _MEDIA_KIND_TO_OCTOAI_IMPORT_NAMES = {
        MediaKind.IMAGE.value: "Image",
        MediaKind.VIDEO.value: "Video",
        MediaKind.AUDIO.value: "Audio",
    }

    def generate(self, service: InferenceService) -> MustacheGenerationResult:
        """Generate usage example given the parsed ``InferenceService``.

        :param service: the parsed ``InferenceService``.
        """
        generation_template = self._get_generation_template()

        input_result = self._generate_input(service.inference_endpoint.inputs)
        output_result = self._generate_output(service.inference_endpoint.output)
        imports_result = self._generate_imports(
            input_result.discovered_special_types
            | output_result.discovered_special_types
        )

        full_result = generation_template.format(
            input_placeholder=input_result.generated_template,
            output_placeholder=output_result.generated_string,
            imports_placeholder=imports_result,
        )

        return MustacheGenerationResult(
            generated_template=full_result,
            template_hash={
                "endpoint": "<endpoint>",
                "access_token": "<octoai_token>",
                **input_result.template_hash,
            },
        )

    def _get_generation_template(self) -> str:
        return (
            """import argparse

"""
            # Put the imports placeholder at the end. We will add a new line if
            # there is any import. (This is so that we maintain two newlines between
            # the import lines and the first line of code.
            """from octoai.client import Client{imports_placeholder}


inputs = {input_placeholder}


def main(endpoint):
    \"\"\"Run inference against the endpoint.\"\"\"
    # create an OctoAI client
    client = Client({{{{#access_token}}}}token={{{{{{access_token}}}}}}"""
            r"""{{{{/access_token}}}})

    # perform inference
    response = client.infer(endpoint_url=f"{{endpoint}}/infer", inputs=inputs)
    output = response["output"]

    print({output_placeholder})


if __name__ = "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--endpoint", type=str, default="{{{{{{endpoint}}}}}}")

    args = parser.parse_args()
    main(args.endpoint)
"""
        )

    def _generate_input(self, field: Field) -> PythonMustacheGenerationResult:
        generator = PythonInputGenerator()
        result = generator.generate(field)
        return cast(PythonMustacheGenerationResult, result)

    def _generate_output(self, field: Field) -> PythonOutputGenerationResult:
        generator = PythonOutputGenerator()
        result = generator.generate(field)
        return result

    def _generate_imports(self, discovered_types: Set[SpecialType]) -> str:
        octoai_types = []

        for discovered_type in discovered_types:
            if type(discovered_type) is MediaType:
                octoai_types.append(
                    PythonExampleGenerator._MEDIA_KIND_TO_OCTOAI_IMPORT_NAMES[
                        cast(MediaType, discovered_type).kind.value
                    ]
                )

        imports = []

        if octoai_types:
            imports += [f'from octoai.types import {", ".join(octoai_types)}']

        if imports:
            return "\n" + "\n".join(imports)
        else:
            return ""



# coding: utf-8

"""
    fine-tuning-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic.v1 import BaseModel

class NullTune(BaseModel):
    """
    A null tune so the tagged union works.  # noqa: E501
    """
    tune_type: Optional[Any] = None
    __properties = ["tune_type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NullTune:
        """Create an instance of NullTune from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if tune_type (nullable) is None
        # and __fields_set__ contains the field
        if self.tune_type is None and "tune_type" in self.__fields_set__:
            _dict['tune_type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> NullTune:
        """Create an instance of NullTune from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return NullTune.parse_obj(obj)

        _obj = NullTune.parse_obj({
            "tune_type": obj.get("tune_type")
        })
        return _obj





# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import re  # noqa: F401
import io
import warnings

from pydantic.v1 import validate_arguments, ValidationError

from pydantic.v1 import StrictBool, StrictInt, StrictStr, conlist

from typing import Optional

from clients.asset_lake.models.asset_type import AssetType
from clients.asset_lake.models.base_engine import BaseEngine
from clients.asset_lake.models.complete_asset_upload_request import CompleteAssetUploadRequest
from clients.asset_lake.models.complete_asset_upload_response import CompleteAssetUploadResponse
from clients.asset_lake.models.create_asset_request import CreateAssetRequest
from clients.asset_lake.models.create_asset_response import CreateAssetResponse
from clients.asset_lake.models.data_type import DataType
from clients.asset_lake.models.delete_asset_response import DeleteAssetResponse
from clients.asset_lake.models.list_assets_response import ListAssetsResponse
from clients.asset_lake.models.retrieve_asset_response import RetrieveAssetResponse
from clients.asset_lake.models.transfer_api_type import TransferApiType

from clients.asset_lake.api_client import ApiClient
from clients.asset_lake.api_response import ApiResponse
from clients.asset_lake.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class AssetsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def complete_asset_upload_v1_assets_asset_id_complete_upload_post(self, asset_id : StrictStr, complete_asset_upload_request : CompleteAssetUploadRequest, **kwargs) -> CompleteAssetUploadResponse:  # noqa: E501
        """Complete Asset Upload  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.complete_asset_upload_v1_assets_asset_id_complete_upload_post(asset_id, complete_asset_upload_request, async_req=True)
        >>> result = thread.get()

        :param asset_id: (required)
        :type asset_id: str
        :param complete_asset_upload_request: (required)
        :type complete_asset_upload_request: CompleteAssetUploadRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CompleteAssetUploadResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the complete_asset_upload_v1_assets_asset_id_complete_upload_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.complete_asset_upload_v1_assets_asset_id_complete_upload_post_with_http_info(asset_id, complete_asset_upload_request, **kwargs)  # noqa: E501

    @validate_arguments
    def complete_asset_upload_v1_assets_asset_id_complete_upload_post_with_http_info(self, asset_id : StrictStr, complete_asset_upload_request : CompleteAssetUploadRequest, **kwargs) -> ApiResponse:  # noqa: E501
        """Complete Asset Upload  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.complete_asset_upload_v1_assets_asset_id_complete_upload_post_with_http_info(asset_id, complete_asset_upload_request, async_req=True)
        >>> result = thread.get()

        :param asset_id: (required)
        :type asset_id: str
        :param complete_asset_upload_request: (required)
        :type complete_asset_upload_request: CompleteAssetUploadRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CompleteAssetUploadResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'asset_id',
            'complete_asset_upload_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method complete_asset_upload_v1_assets_asset_id_complete_upload_post" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['asset_id']:
            _path_params['asset_id'] = _params['asset_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['complete_asset_upload_request'] is not None:
            _body_params = _params['complete_asset_upload_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "CompleteAssetUploadResponse",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/v1/assets/{asset_id}/complete-upload', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def create_asset_v1_assets_post(self, create_asset_request : CreateAssetRequest, **kwargs) -> CreateAssetResponse:  # noqa: E501
        """Create Asset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_asset_v1_assets_post(create_asset_request, async_req=True)
        >>> result = thread.get()

        :param create_asset_request: (required)
        :type create_asset_request: CreateAssetRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CreateAssetResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the create_asset_v1_assets_post_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.create_asset_v1_assets_post_with_http_info(create_asset_request, **kwargs)  # noqa: E501

    @validate_arguments
    def create_asset_v1_assets_post_with_http_info(self, create_asset_request : CreateAssetRequest, **kwargs) -> ApiResponse:  # noqa: E501
        """Create Asset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.create_asset_v1_assets_post_with_http_info(create_asset_request, async_req=True)
        >>> result = thread.get()

        :param create_asset_request: (required)
        :type create_asset_request: CreateAssetRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CreateAssetResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'create_asset_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_asset_v1_assets_post" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['create_asset_request'] is not None:
            _body_params = _params['create_asset_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "CreateAssetResponse",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/v1/assets', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def delete_asset_v1_assets_asset_id_delete(self, asset_id : StrictStr, **kwargs) -> DeleteAssetResponse:  # noqa: E501
        """Delete Asset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_asset_v1_assets_asset_id_delete(asset_id, async_req=True)
        >>> result = thread.get()

        :param asset_id: (required)
        :type asset_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: DeleteAssetResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the delete_asset_v1_assets_asset_id_delete_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.delete_asset_v1_assets_asset_id_delete_with_http_info(asset_id, **kwargs)  # noqa: E501

    @validate_arguments
    def delete_asset_v1_assets_asset_id_delete_with_http_info(self, asset_id : StrictStr, **kwargs) -> ApiResponse:  # noqa: E501
        """Delete Asset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.delete_asset_v1_assets_asset_id_delete_with_http_info(asset_id, async_req=True)
        >>> result = thread.get()

        :param asset_id: (required)
        :type asset_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(DeleteAssetResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'asset_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_asset_v1_assets_asset_id_delete" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['asset_id']:
            _path_params['asset_id'] = _params['asset_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "DeleteAssetResponse",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/v1/assets/{asset_id}', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def list_assets_v1_assets_get(self, name : Optional[StrictStr] = None, is_public : Optional[StrictBool] = None, data_type : Optional[DataType] = None, asset_type : Optional[conlist(AssetType)] = None, engine : Optional[conlist(BaseEngine)] = None, limit : Optional[StrictInt] = None, offset : Optional[StrictInt] = None, get_preview_urls : Optional[StrictBool] = None, asset_ids : Optional[StrictStr] = None, owner : Optional[StrictStr] = None, **kwargs) -> ListAssetsResponse:  # noqa: E501
        """List Assets  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_assets_v1_assets_get(name, is_public, data_type, asset_type, engine, limit, offset, get_preview_urls, asset_ids, owner, async_req=True)
        >>> result = thread.get()

        :param name:
        :type name: str
        :param is_public:
        :type is_public: bool
        :param data_type:
        :type data_type: DataType
        :param asset_type:
        :type asset_type: List[AssetType]
        :param engine:
        :type engine: List[BaseEngine]
        :param limit:
        :type limit: int
        :param offset:
        :type offset: int
        :param get_preview_urls:
        :type get_preview_urls: bool
        :param asset_ids:
        :type asset_ids: str
        :param owner:
        :type owner: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ListAssetsResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the list_assets_v1_assets_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.list_assets_v1_assets_get_with_http_info(name, is_public, data_type, asset_type, engine, limit, offset, get_preview_urls, asset_ids, owner, **kwargs)  # noqa: E501

    @validate_arguments
    def list_assets_v1_assets_get_with_http_info(self, name : Optional[StrictStr] = None, is_public : Optional[StrictBool] = None, data_type : Optional[DataType] = None, asset_type : Optional[conlist(AssetType)] = None, engine : Optional[conlist(BaseEngine)] = None, limit : Optional[StrictInt] = None, offset : Optional[StrictInt] = None, get_preview_urls : Optional[StrictBool] = None, asset_ids : Optional[StrictStr] = None, owner : Optional[StrictStr] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """List Assets  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_assets_v1_assets_get_with_http_info(name, is_public, data_type, asset_type, engine, limit, offset, get_preview_urls, asset_ids, owner, async_req=True)
        >>> result = thread.get()

        :param name:
        :type name: str
        :param is_public:
        :type is_public: bool
        :param data_type:
        :type data_type: DataType
        :param asset_type:
        :type asset_type: List[AssetType]
        :param engine:
        :type engine: List[BaseEngine]
        :param limit:
        :type limit: int
        :param offset:
        :type offset: int
        :param get_preview_urls:
        :type get_preview_urls: bool
        :param asset_ids:
        :type asset_ids: str
        :param owner:
        :type owner: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ListAssetsResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'name',
            'is_public',
            'data_type',
            'asset_type',
            'engine',
            'limit',
            'offset',
            'get_preview_urls',
            'asset_ids',
            'owner'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_assets_v1_assets_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('name') is not None:  # noqa: E501
            _query_params.append(('name', _params['name']))

        if _params.get('is_public') is not None:  # noqa: E501
            _query_params.append(('is_public', _params['is_public']))

        if _params.get('data_type') is not None:  # noqa: E501
            _query_params.append(('data_type', _params['data_type'].value))

        if _params.get('asset_type') is not None:  # noqa: E501
            _query_params.append(('asset_type', _params['asset_type']))
            _collection_formats['asset_type'] = 'multi'

        if _params.get('engine') is not None:  # noqa: E501
            _query_params.append(('engine', _params['engine']))
            _collection_formats['engine'] = 'multi'

        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        if _params.get('offset') is not None:  # noqa: E501
            _query_params.append(('offset', _params['offset']))

        if _params.get('get_preview_urls') is not None:  # noqa: E501
            _query_params.append(('get_preview_urls', _params['get_preview_urls']))

        if _params.get('asset_ids') is not None:  # noqa: E501
            _query_params.append(('asset_ids', _params['asset_ids']))

        if _params.get('owner') is not None:  # noqa: E501
            _query_params.append(('owner', _params['owner']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "ListAssetsResponse",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/v1/assets', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def retrieve_asset_v1_assets_asset_owner_and_name_or_id_get(self, asset_owner_and_name_or_id : StrictStr, transfer_api_type : Optional[TransferApiType] = None, **kwargs) -> RetrieveAssetResponse:  # noqa: E501
        """Retrieve Asset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.retrieve_asset_v1_assets_asset_owner_and_name_or_id_get(asset_owner_and_name_or_id, transfer_api_type, async_req=True)
        >>> result = thread.get()

        :param asset_owner_and_name_or_id: (required)
        :type asset_owner_and_name_or_id: str
        :param transfer_api_type:
        :type transfer_api_type: TransferApiType
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: RetrieveAssetResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the retrieve_asset_v1_assets_asset_owner_and_name_or_id_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.retrieve_asset_v1_assets_asset_owner_and_name_or_id_get_with_http_info(asset_owner_and_name_or_id, transfer_api_type, **kwargs)  # noqa: E501

    @validate_arguments
    def retrieve_asset_v1_assets_asset_owner_and_name_or_id_get_with_http_info(self, asset_owner_and_name_or_id : StrictStr, transfer_api_type : Optional[TransferApiType] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Retrieve Asset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.retrieve_asset_v1_assets_asset_owner_and_name_or_id_get_with_http_info(asset_owner_and_name_or_id, transfer_api_type, async_req=True)
        >>> result = thread.get()

        :param asset_owner_and_name_or_id: (required)
        :type asset_owner_and_name_or_id: str
        :param transfer_api_type:
        :type transfer_api_type: TransferApiType
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(RetrieveAssetResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'asset_owner_and_name_or_id',
            'transfer_api_type'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method retrieve_asset_v1_assets_asset_owner_and_name_or_id_get" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['asset_owner_and_name_or_id']:
            _path_params['asset_owner_and_name_or_id'] = _params['asset_owner_and_name_or_id']


        # process the query parameters
        _query_params = []
        if _params.get('transfer_api_type') is not None:  # noqa: E501
            _query_params.append(('transfer_api_type', _params['transfer_api_type'].value))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            '200': "RetrieveAssetResponse",
            '422': "HTTPValidationError",
        }

        return self.api_client.call_api(
            '/v1/assets/{asset_owner_and_name_or_id}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    fine-tuning-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, List, Optional
from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr, conlist
from clients.fine_tuning.models.lora_tune_checkpoint import LoraTuneCheckpoint
from clients.fine_tuning.models.lora_tune_file import LoraTuneFile

class LoraTuneInput(BaseModel):
    """
    A LoRA Tune.  # noqa: E501
    """
    base_checkpoint: LoraTuneCheckpoint = Field(...)
    files: conlist(LoraTuneFile) = Field(..., description="The files used for this LoRA Tune.")
    seed: Optional[StrictInt] = None
    steps: StrictInt = Field(..., description="The number of steps used for this LoRA Tune.")
    trigger_words: conlist(StrictStr) = Field(..., description="The trigger words used for this tune. As of now, only one trigger word is supported. `List` is used for future-proofing.")
    tune_type: Optional[Any] = Field(None, description="Tune type.")
    __properties = ["base_checkpoint", "files", "seed", "steps", "trigger_words", "tune_type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> LoraTuneInput:
        """Create an instance of LoraTuneInput from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of base_checkpoint
        if self.base_checkpoint:
            _dict['base_checkpoint'] = self.base_checkpoint.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in files (list)
        _items = []
        if self.files:
            for _item in self.files:
                if _item:
                    _items.append(_item.to_dict())
            _dict['files'] = _items
        # set to None if seed (nullable) is None
        # and __fields_set__ contains the field
        if self.seed is None and "seed" in self.__fields_set__:
            _dict['seed'] = None

        # set to None if tune_type (nullable) is None
        # and __fields_set__ contains the field
        if self.tune_type is None and "tune_type" in self.__fields_set__:
            _dict['tune_type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> LoraTuneInput:
        """Create an instance of LoraTuneInput from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return LoraTuneInput.parse_obj(obj)

        _obj = LoraTuneInput.parse_obj({
            "base_checkpoint": LoraTuneCheckpoint.from_dict(obj.get("base_checkpoint")) if obj.get("base_checkpoint") is not None else None,
            "files": [LoraTuneFile.from_dict(_item) for _item in obj.get("files")] if obj.get("files") is not None else None,
            "seed": obj.get("seed"),
            "steps": obj.get("steps"),
            "trigger_words": obj.get("trigger_words"),
            "tune_type": obj.get("tune_type")
        })
        return _obj





from __future__ import annotations
# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from enum import Enum





class Scheduler(str, Enum):
    """
    The schedulers available for image generation.
    """

    """
    allowed enum values
    """
    PNDM = 'PNDM'
    LMS = 'LMS'
    KLMS = 'KLMS'
    DDIM = 'DDIM'
    DDPM = 'DDPM'
    HEUN = 'HEUN'
    K_HEUN = 'K_HEUN'
    K_EULER = 'K_EULER'
    K_EULER_ANCESTRAL = 'K_EULER_ANCESTRAL'
    DPM_SOLVER_MULTISTEP = 'DPM_SOLVER_MULTISTEP'
    DPM_PLUS_PLUS_2_M_KARRAS = 'DPM_PLUS_PLUS_2M_KARRAS'
    DPM_SINGLE = 'DPM_SINGLE'
    DPM_2 = 'DPM_2'
    DPM_2_ANCESTRAL = 'DPM_2_ANCESTRAL'
    DPM_PLUS_PLUS_SDE_KARRAS = 'DPM_PLUS_PLUS_SDE_KARRAS'
    UNI_PC = 'UNI_PC'
    LCM = 'LCM'

    @classmethod
    def from_json(cls, json_str: str) -> Scheduler:
        """Create an instance of Scheduler from a JSON string"""
        return Scheduler(json.loads(json_str))





# coding: utf-8

# flake8: noqa
"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from clients.ollm.models.chat_completion_request_ext import ChatCompletionRequestExt
from clients.ollm.models.chat_completion_request_ext_vllm import ChatCompletionRequestExtVLLM
from clients.ollm.models.chat_fn_call import ChatFnCall
from clients.ollm.models.chat_message import ChatMessage
from clients.ollm.models.create_chat_completion_request import CreateChatCompletionRequest
from clients.ollm.models.function import Function
from clients.ollm.models.function_call import FunctionCall
from clients.ollm.models.http_validation_error import HTTPValidationError
from clients.ollm.models.location_inner import LocationInner
from clients.ollm.models.stop import Stop
from clients.ollm.models.validation_error import ValidationError



# coding: utf-8

"""
    fine-tuning-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from enum import Enum





class TuneStatus(str, Enum):
    """
    The status of the tune job.
    """

    """
    allowed enum values
    """
    PENDING = 'pending'
    RUNNING = 'running'
    SUCCEEDED = 'succeeded'
    CANCELLED = 'cancelled'
    FAILED = 'failed'

    @classmethod
    def from_json(cls, json_str: str) -> TuneStatus:
        """Create an instance of TuneStatus from a JSON string"""
        return TuneStatus(json.loads(json_str))





"""
Entities and APIs for working with text generation models.

Instead of using these classes directly, developers should
use the octoai.client.Client class. For example:

client = octoai.client.Client()
completion = client.chat.completions.create(...)
"""

from enum import Enum
from typing import Iterable, List, Optional, Union

from pydantic.v1 import BaseModel
from pydantic.v1.error_wrappers import ValidationError
from typing_extensions import Literal

from clients.ollm.models.chat_message import ChatMessage
from clients.ollm.models.create_chat_completion_request import (
    CreateChatCompletionRequest,
)
from clients.ollm.models.stop import Stop
from octoai.client import Client
from octoai.errors import OctoAIValidationError


class TextModel(str, Enum):
    """List of available text models."""

    LLAMA_2_13B_CHAT_FP16 = "llama-2-13b-chat-fp16"
    LLAMA_2_70B_CHAT_FP16 = "llama-2-70b-chat-fp16"
    LLAMA_2_70B_CHAT_INT4 = "llama-2-70b-chat-int4"
    CODELLAMA_7B_INSTRUCT_FP16 = "codellama-7b-instruct-fp16"
    CODELLAMA_13B_INSTRUCT_FP16 = "codellama-13b-instruct-fp16"
    CODELLAMA_34B_INSTRUCT_FP16 = "codellama-34b-instruct-fp16"
    CODELLAMA_34B_INSTRUCT_INT4 = "codellama-34b-instruct-int4"
    MISTRAL_7B_INSTRUCT_FP16 = "mistral-7b-instruct-fp16"
    MIXTRAL_8X7B_INSTRUCT_FP16 = "mixtral-8x7b-instruct-fp16"

    def to_name(self):
        """Return the name of the model."""
        if self == self.LLAMA_2_13B_CHAT_FP16:
            return "llama-2-13b-chat-fp16"
        elif self == self.LLAMA_2_70B_CHAT_FP16:
            return "llama-2-70b-chat-fp16"
        elif self == self.LLAMA_2_70B_CHAT_INT4:
            return "llama-2-70b-chat-int4"
        elif self == self.CODELLAMA_7B_INSTRUCT_FP16:
            return "codellama-7b-instruct-fp16"
        elif self == self.CODELLAMA_13B_INSTRUCT_FP16:
            return "codellama-13b-instruct-fp16"
        elif self == self.MISTRAL_7B_INSTRUCT_FP16:
            return "mistral-7b-instruct-fp16"
        elif self == self.MIXTRAL_8X7B_INSTRUCT_FP16:
            return "mixtral-8x7b-instruct-fp16"


def get_model_list() -> List[str]:
    """Return a list of available text models."""
    return [model.value for model in TextModel]


class ChoiceDelta(BaseModel):
    """Contents for streaming text completion responses."""

    content: Optional[str] = None
    role: Optional[Literal["system", "user", "assistant", "tool"]] = None


class Choice(BaseModel):
    """A single choice in a text completion response."""

    index: int
    message: ChatMessage = None
    delta: ChoiceDelta = None
    finish_reason: Literal[
        "stop", "length", "tool_calls", "content_filter", "function_call"
    ] = None


class CompletionUsage(BaseModel):
    """Usage statistics for a text completion response."""

    completion_tokens: int
    prompt_tokens: int
    total_tokens: int


class ChatCompletion(BaseModel):
    """A text completion response."""

    id: str
    choices: List[Choice]
    created: int
    model: str
    object: Optional[Literal["chat.completion", "chat.completion.chunk"]] = None
    system_fingerprint: Optional[str] = None
    usage: Optional[CompletionUsage] = None


class Completions:
    """Text completions API."""

    client: Client
    endpoint: str = "https://text.octoai.run/v1/chat/completions"

    def __init__(self, client: Client) -> None:
        self.client = client

    def create(
        self,
        *,
        messages: List[ChatMessage],
        model: Union[str, TextModel],
        frequency_penalty: Optional[float] = None,
        max_tokens: Optional[int] = None,
        presence_penalty: Optional[float] = 0.0,
        stop: Optional[Stop] = None,
        stream: Optional[bool] = False,
        temperature: Optional[float] = 1.0,
        top_p: Optional[float] = 1.0,
    ) -> Union[ChatCompletion, Iterable[ChatCompletion]]:
        """
        Create a chat completion with a text generation model.

        :param messages: Required. A list of messages to use as context for the
            completion.
        :param model: Required. The model to use for the completion. Supported models
            are listed in the `octoai.chat.TextModel` enum.
        :param frequency_penalty: Positive values make it less likely that the model
            repeats tokens several times in the completion. Valid values are between
            -2.0 and 2.0.
        :param max_tokens: The maximum number of tokens to generate.
        :param presence_penalty: Positive values make it less likely that the model
            repeats tokens in the completion. Valid values are between -2.0 and 2.0.
        :param stop: A list of sequences where the model stops generating tokens.
        :param stream: Whether to return a generator that yields partial message
            deltas as they become available, instead of waiting to return the entire
            response.
        :param temperature: Sampling temperature. A value between 0 and 2. Higher values
            make the model more creative by sampling less likely tokens.
        :param top_p: The cumulative probability of the most likely tokens to use. Use
            `temperature` or `top_p` but not both.
        """
        request = CreateChatCompletionRequest(
            messages=messages,
            model=model.value if isinstance(model, TextModel) else model,
            frequency_penalty=frequency_penalty,
            function_call=None,
            functions=None,
            logit_bias=None,
            max_tokens=max_tokens,
            n=1,
            presence_penalty=presence_penalty,
            stop=stop,
            stream=stream,
            temperature=temperature,
            top_p=top_p,
            user=None,
        )

        inputs = request.to_dict()

        if stream:
            return self.client.infer_stream(
                self.endpoint, inputs, map_fn=lambda resp: ChatCompletion(**resp)
            )  # type: ignore

        resp = self.client.infer(self.endpoint, inputs)
        try:
            return ChatCompletion(**resp)
        except ValidationError as e:
            raise OctoAIValidationError(
                "Unable to validate response from server.", caused_by=e
            )


class Chat:
    """Chat API for text generation models."""

    completions: Completions

    def __init__(self, client: Client):
        self.completions = Completions(client)



# coding: utf-8

"""
    fine-tuning-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictStr
from clients.fine_tuning.models.base_engine import BaseEngine

class LoraTuneCheckpoint(BaseModel):
    """
    A checkpoint used for a LoRA Tune.  # noqa: E501
    """
    checkpoint_id: StrictStr = Field(..., description="The checkpoint asset ID.")
    engine: Optional[BaseEngine] = None
    name: Optional[StrictStr] = None
    __properties = ["checkpoint_id", "engine", "name"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> LoraTuneCheckpoint:
        """Create an instance of LoraTuneCheckpoint from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if engine (nullable) is None
        # and __fields_set__ contains the field
        if self.engine is None and "engine" in self.__fields_set__:
            _dict['engine'] = None

        # set to None if name (nullable) is None
        # and __fields_set__ contains the field
        if self.name is None and "name" in self.__fields_set__:
            _dict['name'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> LoraTuneCheckpoint:
        """Create an instance of LoraTuneCheckpoint from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return LoraTuneCheckpoint.parse_obj(obj)

        _obj = LoraTuneCheckpoint.parse_obj({
            "checkpoint_id": obj.get("checkpoint_id"),
            "engine": obj.get("engine"),
            "name": obj.get("name")
        })
        return _obj





# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictBool, StrictInt, StrictStr
from clients.asset_lake.models.asset_type import AssetType
from clients.asset_lake.models.data import Data
from clients.asset_lake.models.status import Status

class Asset(BaseModel):
    """
    Asset
    """
    asset_type: AssetType = Field(...)
    created_at: StrictStr = Field(..., description="Asset creation time.")
    data: Data = Field(...)
    description: StrictStr = Field(..., description="Description.")
    id: StrictStr = Field(..., description="Asset id.")
    is_public: Optional[StrictBool] = Field(False, description="True if asset is public.")
    name: StrictStr = Field(..., description="Asset name.")
    size_bytes: StrictInt = Field(..., description="Size in bytes.")
    status: Status = Field(...)
    status_details: StrictStr = Field(..., description="Asset status.")
    tenant_uuid: StrictStr = Field(..., description="Asset tenant ID.")
    __properties = ["asset_type", "created_at", "data", "description", "id", "is_public", "name", "size_bytes", "status", "status_details", "tenant_uuid"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Asset:
        """Create an instance of Asset from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of data
        if self.data:
            _dict['data'] = self.data.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Asset:
        """Create an instance of Asset from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Asset.parse_obj(obj)

        _obj = Asset.parse_obj({
            "asset_type": obj.get("asset_type"),
            "created_at": obj.get("created_at"),
            "data": Data.from_dict(obj.get("data")) if obj.get("data") is not None else None,
            "description": obj.get("description"),
            "id": obj.get("id"),
            "is_public": obj.get("is_public") if obj.get("is_public") is not None else False,
            "name": obj.get("name"),
            "size_bytes": obj.get("size_bytes"),
            "status": obj.get("status"),
            "status_details": obj.get("status_details"),
            "tenant_uuid": obj.get("tenant_uuid")
        })
        return _obj





"""Initializes the spec module."""

from .ast import (
    BaseType,
    CustomObjectType,
    DictionaryType,
    EnumType,
    Field,
    FieldInfo,
    InferenceEndpoint,
    InferenceService,
    ListType,
    MediaKind,
    MediaType,
    NamedField,
    OptionalType,
    PrimitiveType,
    PydanticHttpUrlType,
    SetType,
    SimpleFieldInfo,
    SpecialType,
    TupleType,
    UnionType,
    UnknownType,
)
from .parser import parse_service

__all__ = [
    "BaseType",
    "CustomObjectType",
    "DictionaryType",
    "EnumType",
    "Field",
    "FieldInfo",
    "InferenceEndpoint",
    "InferenceService",
    "ListType",
    "MediaKind",
    "MediaType",
    "NamedField",
    "OptionalType",
    "PrimitiveType",
    "PydanticHttpUrlType",
    "SetType",
    "SimpleFieldInfo",
    "SpecialType",
    "TupleType",
    "UnionType",
    "UnknownType",
    "parse_service",
]



"""OctoAI Fine-tuning and Assets."""
from __future__ import (
    annotations,  # required to allow 3.7+ python use type | syntax introduced in 3.10
)

from abc import abstractmethod

from octoai.client import Client


class BaseModel(Client):
    """Base model."""

    # Future inclusions:
    # tune: LoraTuneModel
    # extract: LoraExtractionModel
    # checkpoint: CheckpointModel
    #
    # asset: AssetModel

    def __init__(
        self,
        api_endpoint: str,  # Used for dependency injection, otherwise child class
        token: str | None = None,
        *args,
        **kwargs,
    ):
        if api_endpoint[-1] != "/":
            api_endpoint += "/"
        self.api_endpoint = api_endpoint
        super(BaseModel, self).__init__(token)

    @abstractmethod
    def generate(self, *args, **kwargs):
        """Generate.  Only instantiated by child class."""
        pass



from __future__ import annotations
# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from enum import Enum





class AssetType(str, Enum):
    """
    AssetType
    """

    """
    allowed enum values
    """
    FILE = 'file'
    CHECKPOINT = 'checkpoint'
    LORA = 'lora'
    TEXTUAL_INVERSION = 'textual_inversion'
    VAE = 'vae'
    VOLUME = 'volume'

    @classmethod
    def from_json(cls, json_str: str) -> AssetType:
        """Create an instance of AssetType from a JSON string"""
        return AssetType(json.loads(json_str))





"""OctoAI errors."""

from typing import Optional


class OctoAIError(Exception):
    """Base class for all OctoAI errors.

    :param message: error message
    :type message: str
    :param caused_by: Exception source
    :type caused_by: :class:Exception, optional
    """

    def __init__(
        self,
        message: str,
        caused_by: Optional[Exception] = None,
    ):
        """Declare OctoAIError class."""
        self.message = message
        self.caused_by = caused_by

    def __repr__(self) -> str:
        """Format string representation to include wrapped exception message."""
        msg = f"{self.__class__.__name__}({self.message})"
        if self.caused_by is not None:
            msg += f" caused by {self.caused_by.__class__.__name__}({self.caused_by})"
        return msg


class OctoAIClientError(OctoAIError):
    """Raise when the client returns a 4xx error."""


class OctoAIServerError(OctoAIError):
    """Raise when the server returns a 5xx error."""


class OctoAIValidationError(OctoAIError):
    """Raise when inputs are unable to be validated."""



# coding: utf-8

"""
    fine-tuning-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictStr
from clients.fine_tuning.models.details import Details

class CreateTuneRequest(BaseModel):
    """
    The create tune request.  # noqa: E501
    """
    description: Optional[StrictStr] = None
    details: Details = Field(...)
    name: StrictStr = Field(..., description="The name of the tune.")
    __properties = ["description", "details", "name"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateTuneRequest:
        """Create an instance of CreateTuneRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of details
        if self.details:
            _dict['details'] = self.details.to_dict()
        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateTuneRequest:
        """Create an instance of CreateTuneRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateTuneRequest.parse_obj(obj)

        _obj = CreateTuneRequest.parse_obj({
            "description": obj.get("description"),
            "details": Details.from_dict(obj.get("details")) if obj.get("details") is not None else None,
            "name": obj.get("name")
        })
        return _obj





# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional, Union
from pydantic.v1 import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr

class LegacyImageGenerationRequest(BaseModel):
    """
    Model input data type.  # noqa: E501
    """
    prompt: StrictStr = Field(...)
    prompt_2: Optional[StrictStr] = None
    negative_prompt: Optional[StrictStr] = None
    negative_prompt_2: Optional[StrictStr] = None
    model: Optional[StrictStr] = None
    loras: Optional[Any] = None
    sampler: Optional[StrictStr] = 'DDIM'
    height: Optional[StrictInt] = 1024
    width: Optional[StrictInt] = 1024
    cfg_scale: Optional[Union[StrictFloat, StrictInt]] = 12.0
    steps: Optional[StrictInt] = 30
    num_images: Optional[StrictInt] = 1
    seed: Optional[StrictInt] = None
    init_image: Optional[StrictStr] = None
    strength: Optional[Union[StrictFloat, StrictInt]] = 0.8
    style_preset: Optional[StrictStr] = None
    use_refiner: Optional[StrictBool] = True
    high_noise_frac: Optional[Union[StrictFloat, StrictInt]] = 0.8
    enable_safety: Optional[StrictBool] = False
    output_type: Optional[StrictStr] = 'jpg'
    __properties = ["prompt", "prompt_2", "negative_prompt", "negative_prompt_2", "model", "loras", "sampler", "height", "width", "cfg_scale", "steps", "num_images", "seed", "init_image", "strength", "style_preset", "use_refiner", "high_noise_frac", "enable_safety", "output_type"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> LegacyImageGenerationRequest:
        """Create an instance of LegacyImageGenerationRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of loras
        if self.loras:
            _dict['loras'] = self.loras.to_dict()
        # set to None if prompt_2 (nullable) is None
        # and __fields_set__ contains the field
        if self.prompt_2 is None and "prompt_2" in self.__fields_set__:
            _dict['prompt_2'] = None

        # set to None if negative_prompt (nullable) is None
        # and __fields_set__ contains the field
        if self.negative_prompt is None and "negative_prompt" in self.__fields_set__:
            _dict['negative_prompt'] = None

        # set to None if negative_prompt_2 (nullable) is None
        # and __fields_set__ contains the field
        if self.negative_prompt_2 is None and "negative_prompt_2" in self.__fields_set__:
            _dict['negative_prompt_2'] = None

        # set to None if model (nullable) is None
        # and __fields_set__ contains the field
        if self.model is None and "model" in self.__fields_set__:
            _dict['model'] = None

        # set to None if loras (nullable) is None
        # and __fields_set__ contains the field
        if self.loras is None and "loras" in self.__fields_set__:
            _dict['loras'] = None

        # set to None if seed (nullable) is None
        # and __fields_set__ contains the field
        if self.seed is None and "seed" in self.__fields_set__:
            _dict['seed'] = None

        # set to None if init_image (nullable) is None
        # and __fields_set__ contains the field
        if self.init_image is None and "init_image" in self.__fields_set__:
            _dict['init_image'] = None

        # set to None if style_preset (nullable) is None
        # and __fields_set__ contains the field
        if self.style_preset is None and "style_preset" in self.__fields_set__:
            _dict['style_preset'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> LegacyImageGenerationRequest:
        """Create an instance of LegacyImageGenerationRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return LegacyImageGenerationRequest.parse_obj(obj)

        _obj = LegacyImageGenerationRequest.parse_obj({
            "prompt": obj.get("prompt"),
            "prompt_2": obj.get("prompt_2"),
            "negative_prompt": obj.get("negative_prompt"),
            "negative_prompt_2": obj.get("negative_prompt_2"),
            "model": obj.get("model"),
            "loras": AnyOf.from_dict(obj.get("loras")) if obj.get("loras") is not None else None,
            "sampler": obj.get("sampler") if obj.get("sampler") is not None else 'DDIM',
            "height": obj.get("height") if obj.get("height") is not None else 1024,
            "width": obj.get("width") if obj.get("width") is not None else 1024,
            "cfg_scale": obj.get("cfg_scale") if obj.get("cfg_scale") is not None else 12.0,
            "steps": obj.get("steps") if obj.get("steps") is not None else 30,
            "num_images": obj.get("num_images") if obj.get("num_images") is not None else 1,
            "seed": obj.get("seed"),
            "init_image": obj.get("init_image"),
            "strength": obj.get("strength") if obj.get("strength") is not None else 0.8,
            "style_preset": obj.get("style_preset"),
            "use_refiner": obj.get("use_refiner") if obj.get("use_refiner") is not None else True,
            "high_noise_frac": obj.get("high_noise_frac") if obj.get("high_noise_frac") is not None else 0.8,
            "enable_safety": obj.get("enable_safety") if obj.get("enable_safety") is not None else False,
            "output_type": obj.get("output_type") if obj.get("output_type") is not None else 'jpg'
        })
        return _obj





# coding: utf-8

"""
    fine-tuning-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import List, Optional
from pydantic.v1 import BaseModel, Field, StrictStr, conlist
from clients.fine_tuning.models.details1 import Details1
from clients.fine_tuning.models.tune_status import TuneStatus

class Tune(BaseModel):
    """
    A tune base class.  # noqa: E501
    """
    created_at: datetime = Field(..., description="The time this LoRA tune was created.")
    deleted_at: Optional[datetime] = Field(...)
    description: StrictStr = Field(..., description="The description of the tune.")
    details: Details1 = Field(...)
    id: StrictStr = Field(..., description="The ID of the LoRA tune.")
    name: StrictStr = Field(..., description="The name of the tune.")
    output_lora_ids: conlist(StrictStr) = Field(..., description="The output LoRA IDs, if the task was successful.")
    status: TuneStatus = Field(...)
    status_details: StrictStr = Field(..., description="The details of the status, only used when the associated task failed.")
    succeeded_at: Optional[datetime] = Field(...)
    tenant_id: StrictStr = Field(..., description="The tenant who requested the LoRA tune.")
    tune_type: StrictStr = Field(..., description="The type of tune requested.")
    updated_at: datetime = Field(..., description="The time this LoRA tune was updated.")
    __properties = ["created_at", "deleted_at", "description", "details", "id", "name", "output_lora_ids", "status", "status_details", "succeeded_at", "tenant_id", "tune_type", "updated_at"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Tune:
        """Create an instance of Tune from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of details
        if self.details:
            _dict['details'] = self.details.to_dict()
        # set to None if deleted_at (nullable) is None
        # and __fields_set__ contains the field
        if self.deleted_at is None and "deleted_at" in self.__fields_set__:
            _dict['deleted_at'] = None

        # set to None if succeeded_at (nullable) is None
        # and __fields_set__ contains the field
        if self.succeeded_at is None and "succeeded_at" in self.__fields_set__:
            _dict['succeeded_at'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Tune:
        """Create an instance of Tune from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Tune.parse_obj(obj)

        _obj = Tune.parse_obj({
            "created_at": obj.get("created_at"),
            "deleted_at": obj.get("deleted_at"),
            "description": obj.get("description"),
            "details": Details1.from_dict(obj.get("details")) if obj.get("details") is not None else None,
            "id": obj.get("id"),
            "name": obj.get("name"),
            "output_lora_ids": obj.get("output_lora_ids"),
            "status": obj.get("status"),
            "status_details": obj.get("status_details"),
            "succeeded_at": obj.get("succeeded_at"),
            "tenant_id": obj.get("tenant_id"),
            "tune_type": obj.get("tune_type"),
            "updated_at": obj.get("updated_at")
        })
        return _obj





# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, StrictBool, StrictInt, conlist

class ChatCompletionRequestExtVLLM(BaseModel):
    """
    ChatCompletionRequestExtVLLM
    """
    best_of: Optional[StrictInt] = None
    top_k: Optional[StrictInt] = -1
    ignore_eos: Optional[StrictBool] = False
    use_beam_search: Optional[StrictBool] = False
    stop_token_ids: Optional[conlist(StrictInt)] = None
    skip_special_tokens: Optional[StrictBool] = True
    __properties = ["best_of", "top_k", "ignore_eos", "use_beam_search", "stop_token_ids", "skip_special_tokens"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ChatCompletionRequestExtVLLM:
        """Create an instance of ChatCompletionRequestExtVLLM from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ChatCompletionRequestExtVLLM:
        """Create an instance of ChatCompletionRequestExtVLLM from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ChatCompletionRequestExtVLLM.parse_obj(obj)

        _obj = ChatCompletionRequestExtVLLM.parse_obj({
            "best_of": obj.get("best_of"),
            "top_k": obj.get("top_k") if obj.get("top_k") is not None else -1,
            "ignore_eos": obj.get("ignore_eos") if obj.get("ignore_eos") is not None else False,
            "use_beam_search": obj.get("use_beam_search") if obj.get("use_beam_search") is not None else False,
            "stop_token_ids": obj.get("stop_token_ids"),
            "skip_special_tokens": obj.get("skip_special_tokens") if obj.get("skip_special_tokens") is not None else True
        })
        return _obj





# coding: utf-8

# flake8: noqa
"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from clients.asset_lake.models.archiver_volume_storage_ref import ArchiverVolumeStorageRef
from clients.asset_lake.models.asset import Asset
from clients.asset_lake.models.asset_type import AssetType
from clients.asset_lake.models.base_engine import BaseEngine
from clients.asset_lake.models.checkpoint_data import CheckpointData
from clients.asset_lake.models.complete_asset_upload_request import CompleteAssetUploadRequest
from clients.asset_lake.models.complete_asset_upload_response import CompleteAssetUploadResponse
from clients.asset_lake.models.create_asset_request import CreateAssetRequest
from clients.asset_lake.models.create_asset_response import CreateAssetResponse
from clients.asset_lake.models.data import Data
from clients.asset_lake.models.data_type import DataType
from clients.asset_lake.models.delete_asset_response import DeleteAssetResponse
from clients.asset_lake.models.file_data import FileData
from clients.asset_lake.models.file_extension import FileExtension
from clients.asset_lake.models.file_format import FileFormat
from clients.asset_lake.models.http_validation_error import HTTPValidationError
from clients.asset_lake.models.list_assets_response import ListAssetsResponse
from clients.asset_lake.models.location_inner import LocationInner
from clients.asset_lake.models.lora_data import LoraData
from clients.asset_lake.models.no_transfer_api import NoTransferApi
from clients.asset_lake.models.presigned_url_transfer_api import PresignedUrlTransferApi
from clients.asset_lake.models.retrieve_asset_response import RetrieveAssetResponse
from clients.asset_lake.models.status import Status
from clients.asset_lake.models.storage_descriptor import StorageDescriptor
from clients.asset_lake.models.sts_transfer_api import StsTransferApi
from clients.asset_lake.models.textual_inversion_data import TextualInversionData
from clients.asset_lake.models.transfer_api import TransferApi
from clients.asset_lake.models.transfer_api1 import TransferApi1
from clients.asset_lake.models.transfer_api_type import TransferApiType
from clients.asset_lake.models.vae_data import VAEData
from clients.asset_lake.models.validation_error import ValidationError
from clients.asset_lake.models.volume_data import VolumeData



"""Nodes of the parsed AST for the ``octoai.Service`` subclass."""

from dataclasses import dataclass
from enum import Enum
from typing import Dict, List, Optional, Tuple, Type, Union

from pydantic.fields import FieldInfo as PydanticFieldInfo


@dataclass(frozen=True)
class BaseType:
    """Describes a type."""

    pass


@dataclass(frozen=True)
class SpecialType(BaseType):
    """Describes a special type is specially handled, e.g., ``octoai.types.Image``."""

    pass


FieldInfo = Union["SimpleFieldInfo", PydanticFieldInfo]
"""Either a ``SimpleFieldInfo`` or a ``pydantic.fields.FieldInfo"""


@dataclass(frozen=True)
class Field:
    """Describes a field in an object."""

    type: BaseType
    info: FieldInfo


@dataclass(frozen=True)
class NamedField(Field):
    """Describes a named field in an object."""

    name: str


@dataclass(frozen=True)
class SimpleFieldInfo:
    """Contains the info for non-Pydantic fields (e.g., method signature)."""

    default: Optional[str]
    description: Optional[str]


@dataclass(frozen=True)
class PrimitiveType(BaseType):
    """Describes primitive types."""

    type: Union[
        Type[str],
        Type[bool],
        Type[bytes],
        Type[int],
        Type[float],
    ]


@dataclass(frozen=True)
class UnionType(BaseType):
    """Describes types annotated with ``typing.Union``."""

    members: List[BaseType]


@dataclass(frozen=True)
class ListType(BaseType):
    """Describes types annotated with ``typing.List``."""

    member_type: BaseType


@dataclass(frozen=True)
class TupleType(BaseType):
    """Describes types annotated with ``typing.Tuple``."""

    members: List[BaseType]


@dataclass(frozen=True)
class SetType(BaseType):
    """Describes types annotated with ``typing.Set``."""

    member_type: BaseType


@dataclass(frozen=True)
class DictionaryType(BaseType):
    """Describes types annotated with ``typing.Dict``."""

    key_type: BaseType
    value_type: BaseType


@dataclass(frozen=True)
class EnumType(BaseType):
    """Describes enum types."""

    name: str
    variants: List[Tuple[str, Union[int, str]]]


@dataclass(frozen=True)
class UnknownType(BaseType):
    """
    Description for types that are not fully specified.

    Examples: ``typing.List`` without type arguments.
    """

    pass


@dataclass(frozen=True)
class OptionalType(BaseType):
    """Describes types annotated with ``typing.Optional``."""

    some_type: BaseType


@dataclass(frozen=True)
class CustomObjectType(BaseType):
    """Describes custom Pydantic ``BaseModel`` subclasses defined by the user."""

    name: str
    fields: Dict[str, Field]


class MediaKind(Enum):
    """Describes the type of the special ``MediaType``."""

    AUDIO = "audio"
    VIDEO = "video"
    IMAGE = "image"


@dataclass(frozen=True)
class MediaType(SpecialType):
    """Describes a special media type such as ``octoai.types.Image``."""

    kind: MediaKind


@dataclass(frozen=True)
class PydanticHttpUrlType(SpecialType):
    """Describes the HttpUrl type provided by Pydantic."""

    pass


@dataclass(frozen=True)
class InferenceEndpoint:
    """Describes the signature of the inference endpoint."""

    inputs: Field
    output: Field


@dataclass(frozen=True)
class InferenceService:
    """Describes the inference service."""

    inference_endpoint: InferenceEndpoint



# coding: utf-8

# flake8: noqa

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from clients.image_gen.api.default_api import DefaultApi

# import ApiClient
from clients.image_gen.api_response import ApiResponse
from clients.image_gen.api_client import ApiClient
from clients.image_gen.configuration import Configuration
from clients.image_gen.exceptions import OpenApiException
from clients.image_gen.exceptions import ApiTypeError
from clients.image_gen.exceptions import ApiValueError
from clients.image_gen.exceptions import ApiKeyError
from clients.image_gen.exceptions import ApiAttributeError
from clients.image_gen.exceptions import ApiException

# import models into sdk package
from clients.image_gen.models.checkpoint import Checkpoint
from clients.image_gen.models.control_net import ControlNet
from clients.image_gen.models.control_net_image import ControlNetImage
from clients.image_gen.models.dictionary_of_lo_ras import DictionaryOfLoRAs
from clients.image_gen.models.dictionary_of_textual_inversions import DictionaryOfTextualInversions
from clients.image_gen.models.engine import Engine
from clients.image_gen.models.fixed_random_seed import FixedRandomSeed
from clients.image_gen.models.http_validation_error import HTTPValidationError
from clients.image_gen.models.image_encoding import ImageEncoding
from clients.image_gen.models.image_generation import ImageGeneration
from clients.image_gen.models.image_generation_request import ImageGenerationRequest
from clients.image_gen.models.image_generation_response import ImageGenerationResponse
from clients.image_gen.models.initial_image import InitialImage
from clients.image_gen.models.legacy_image_generation_request import LegacyImageGenerationRequest
from clients.image_gen.models.legacy_image_generation_response import LegacyImageGenerationResponse
from clients.image_gen.models.mask_image import MaskImage
from clients.image_gen.models.model_response import ModelResponse
from clients.image_gen.models.negative_prompt import NegativePrompt
from clients.image_gen.models.output_image_height import OutputImageHeight
from clients.image_gen.models.output_image_width import OutputImageWidth
from clients.image_gen.models.pre_defined_styles import PreDefinedStyles
from clients.image_gen.models.sdxl_styles import SDXLStyles
from clients.image_gen.models.scheduler import Scheduler
from clients.image_gen.models.second_input_prompt import SecondInputPrompt
from clients.image_gen.models.second_negative_prompt import SecondNegativePrompt
from clients.image_gen.models.vae import VAE
from clients.image_gen.models.validation_error import ValidationError
from clients.image_gen.models.validation_error_loc_inner import ValidationErrorLocInner



# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr
from clients.asset_lake.models.storage_descriptor import StorageDescriptor

class VolumeData(BaseModel):
    """
    VolumeData
    """
    asset_type: Optional[Any] = Field(None, description="Asset type.")
    deletion_cleanup_done: StrictBool = Field(..., description="Determine if cleanup is finished.")
    mark_for_deletion_time: Optional[StrictStr] = Field(...)
    storage: StorageDescriptor = Field(...)
    version: StrictStr = Field(..., description="Version.")
    __properties = ["asset_type", "deletion_cleanup_done", "mark_for_deletion_time", "storage", "version"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> VolumeData:
        """Create an instance of VolumeData from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of storage
        if self.storage:
            _dict['storage'] = self.storage.to_dict()
        # set to None if asset_type (nullable) is None
        # and __fields_set__ contains the field
        if self.asset_type is None and "asset_type" in self.__fields_set__:
            _dict['asset_type'] = None

        # set to None if mark_for_deletion_time (nullable) is None
        # and __fields_set__ contains the field
        if self.mark_for_deletion_time is None and "mark_for_deletion_time" in self.__fields_set__:
            _dict['mark_for_deletion_time'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> VolumeData:
        """Create an instance of VolumeData from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return VolumeData.parse_obj(obj)

        _obj = VolumeData.parse_obj({
            "asset_type": obj.get("asset_type"),
            "deletion_cleanup_done": obj.get("deletion_cleanup_done"),
            "mark_for_deletion_time": obj.get("mark_for_deletion_time"),
            "storage": StorageDescriptor.from_dict(obj.get("storage")) if obj.get("storage") is not None else None,
            "version": obj.get("version")
        })
        return _obj





from __future__ import annotations
# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from enum import Enum





class SDXLStyles(str, Enum):
    """
    The supported SDXL styles.
    """

    """
    allowed enum values
    """
    BASE = 'base'
    ENUM_3D_MINUS_MODEL = '3d-model'
    ANALOG_MINUS_FILM = 'analog-film'
    ANIME = 'anime'
    CINEMATIC = 'cinematic'
    COMIC_MINUS_BOOK = 'comic-book'
    CRAFT_CLAY = 'Craft Clay'
    MODELING_MINUS_COMPOUND = 'modeling-compound'
    DIGITAL_MINUS_ART = 'digital-art'
    ENHANCE = 'enhance'
    FANTASY_MINUS_ART = 'fantasy-art'
    ISOMETRIC = 'isometric'
    LINE_MINUS_ART = 'line-art'
    LOW_MINUS_POLY = 'low-poly'
    NEON_MINUS_PUNK = 'neon-punk'
    ORIGAMI = 'origami'
    PHOTOGRAPHIC = 'photographic'
    PIXEL_MINUS_ART = 'pixel-art'
    TILE_MINUS_TEXTURE = 'tile-texture'
    ADVERTISING = 'Advertising'
    FOOD_PHOTOGRAPHY = 'Food Photography'
    REAL_ESTATE = 'Real Estate'
    ABSTRACT = 'Abstract'
    CUBIST = 'Cubist'
    GRAFFITI = 'Graffiti'
    HYPERREALISM = 'Hyperrealism'
    IMPRESSIONIST = 'Impressionist'
    POINTILLISM = 'Pointillism'
    POP_ART = 'Pop Art'
    PSYCHEDELIC = 'Psychedelic'
    RENAISSANCE = 'Renaissance'
    STEAMPUNK = 'Steampunk'
    SURREALIST = 'Surrealist'
    TYPOGRAPHY = 'Typography'
    WATERCOLOR = 'Watercolor'
    FIGHTING_GAME = 'Fighting Game'
    GTA = 'GTA'
    SUPER_MARIO = 'Super Mario'
    MINECRAFT = 'Minecraft'
    POKÉMON = 'Pokémon'
    RETRO_ARCADE = 'Retro Arcade'
    RETRO_GAME = 'Retro Game'
    RPG_FANTASY_GAME = 'RPG Fantasy Game'
    STRATEGY_GAME = 'Strategy Game'
    STREET_FIGHTER = 'Street Fighter'
    LEGEND_OF_ZELDA = 'Legend of Zelda'
    ARCHITECTURAL = 'Architectural'
    DISCO = 'Disco'
    DREAMSCAPE = 'Dreamscape'
    DYSTOPIAN = 'Dystopian'
    FAIRY_TALE = 'Fairy Tale'
    GOTHIC = 'Gothic'
    GRUNGE = 'Grunge'
    HORROR = 'Horror'
    MINIMALIST = 'Minimalist'
    MONOCHROME = 'Monochrome'
    NAUTICAL = 'Nautical'
    SPACE = 'Space'
    STAINED_GLASS = 'Stained Glass'
    TECHWEAR_FASHION = 'Techwear Fashion'
    TRIBAL = 'Tribal'
    ZENTANGLE = 'Zentangle'
    COLLAGE = 'Collage'
    FLAT_PAPERCUT = 'Flat Papercut'
    KIRIGAMI = 'Kirigami'
    PAPER_MACHE = 'Paper Mache'
    PAPER_QUILLING = 'Paper Quilling'
    PAPERCUT_COLLAGE = 'Papercut Collage'
    PAPERCUT_SHADOW_BOX = 'Papercut Shadow Box'
    STACKED_PAPERCUT = 'Stacked Papercut'
    THICK_LAYERED_PAPERCUT = 'Thick Layered Papercut'
    ALIEN = 'Alien'
    FILM_NOIR = 'Film Noir'
    HDR = 'HDR'
    LONG_EXPOSURE = 'Long Exposure'
    NEON_NOIR = 'Neon Noir'
    SILHOUETTE = 'Silhouette'
    TILT_MINUS_SHIFT = 'Tilt-Shift'

    @classmethod
    def from_json(cls, json_str: str) -> SDXLStyles:
        """Create an instance of SDXLStyles from a JSON string"""
        return SDXLStyles(json.loads(json_str))





# coding: utf-8

# flake8: noqa

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from clients.ollm.api.default_api import DefaultApi

# import ApiClient
from clients.ollm.api_response import ApiResponse
from clients.ollm.api_client import ApiClient
from clients.ollm.configuration import Configuration
from clients.ollm.exceptions import OpenApiException
from clients.ollm.exceptions import ApiTypeError
from clients.ollm.exceptions import ApiValueError
from clients.ollm.exceptions import ApiKeyError
from clients.ollm.exceptions import ApiAttributeError
from clients.ollm.exceptions import ApiException

# import models into sdk package
from clients.ollm.models.chat_completion_request_ext import ChatCompletionRequestExt
from clients.ollm.models.chat_completion_request_ext_vllm import ChatCompletionRequestExtVLLM
from clients.ollm.models.chat_fn_call import ChatFnCall
from clients.ollm.models.chat_message import ChatMessage
from clients.ollm.models.create_chat_completion_request import CreateChatCompletionRequest
from clients.ollm.models.function import Function
from clients.ollm.models.function_call import FunctionCall
from clients.ollm.models.http_validation_error import HTTPValidationError
from clients.ollm.models.location_inner import LocationInner
from clients.ollm.models.stop import Stop
from clients.ollm.models.validation_error import ValidationError
